// ----------------------------------------------------------------------------
// Exams.java
// analyzing the grades of exams
// 
// Author:  Wolfgang Schreiner <Wolfgang.Schreiner@fhs-hagenberg.ac.at>
// Created: August 23, 2001
// ----------------------------------------------------------------------------
public class Exams2
{
  // --------------------------------------------------------------------------
  // analyze the grades of multiple exams; read the grades from the standard
  // input stream, writes the result to the standard output stream.
  // for details, see 'analyzeExamGrades'.
  // --------------------------------------------------------------------------
  public static void main(String args[])
  {
    analyzeExamGrades();
  }

  // --------------------------------------------------------------------------
  // the (intermediate) result of analyzing the grades of an exam
  // --------------------------------------------------------------------------
  public static class Result
  {
    int count = 0;               // number of grades processes so far
    int sum = 0;                 // sum of grades processed so far
    int min = Integer.MAX_VALUE; // minimum grade processed so far
    int max = Integer.MIN_VALUE; // maximum grade processed so far
  }

  // --------------------------------------------------------------------------
  // user repeatedly enters grades from 1 to 5 or 0 to denote the end of exam.
  // system prints number of grades, best, worst and average grade and
  // asks whether to proceed with another exam.
  // --------------------------------------------------------------------------
  public static void analyzeExamGrades()
  {
    char answer = 0;
    do
    {
      Result result = analyzeExam();
      printResult(result);
      answer = askToContinue();
    }
    while (answer == 'y');
  }

  // --------------------------------------------------------------------------
  // iteratively reads and analyzes the grades of an exam.
  // when a grade 0 is read, the result of the analysis is returned.
  // --------------------------------------------------------------------------
  public static Result analyzeExam()
  {
    Result result = new Result();

    // read and process exam
    while (true)
    {
      int grade = readGrade(result.count);
      if (grade == 0) return result;
      processGrade(grade, result);
    }
  }


  // --------------------------------------------------------------------------
  // asks the user for grade number 'count' (starting with 0).
  // if the user enters an integer number from 0 to 5, this is returned as
  // the result. otherwise, the user is asked again.
  // aborts execution, if the input stream has terminated.
  // --------------------------------------------------------------------------
  public static int readGrade(int count)
  {
    while(true)
    {
      System.out.print("Enter grade "+(count+1)+" (0 when done): ");
      int grade = Input.readInt();
      if (Input.isOkay() && 0 <= grade && grade <= 5)
        return grade;
      if (Input.hasEnded())
      {
        System.out.println("Unexpected end of input!");
        System.exit(-1);
      }
      System.out.println("Invalid input!");
      Input.clearError();
    }
  }

  // --------------------------------------------------------------------------
  // processes 'grade' and updates the intermediate 'result'
  // --------------------------------------------------------------------------
  public static void processGrade(int grade, Result result)
  {
    result.count++;
    result.sum += grade;
    if (grade < result.min) result.min = grade;
    if (grade > result.max) result.max = grade;
  }

  // --------------------------------------------------------------------------
  // prints the result of an analysis
  // --------------------------------------------------------------------------
  public static void printResult(Result result)
  {
    System.out.println("\back{}nNumber of grades: " + result.count);

    // these values are only valid, if there was at least one grade 
    if (result.count > 0)
    {
      System.out.println("Best grade: " + result.min);
      System.out.println("Worst grade: " + result.max);
      System.out.println("Average grade: " + 
          (float)result.sum/(float)result.count);
    }

    System.out.println("");
  }

  // --------------------------------------------------------------------------
  // asks user whether to continue. if the user answers 'y' or 'n', this
  // is returned as the result. otherwise, the user is asked again.
  // aborts execution, if the input stream has terminated.
  // --------------------------------------------------------------------------
  public static char askToContinue()
  {    
    while(true)
    {
      System.out.print("Another exam (y/n)? ");             
      char answer = Input.readChar();
      if (Input.isOkay() && (answer == 'y' || answer == 'n'))
        return answer;
      if (Input.hasEnded())
      {
        System.out.println("Unexpected end of input!");
        System.exit(-1);
      }
      System.out.println("Invalid input!");
    }      
  }
}
